package fr.sanchezm.attestationsCovid19.ui.add

import android.annotation.SuppressLint
import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.content.Context
import android.os.Build
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import fr.sanchezm.attestationsCovid19.R
import fr.sanchezm.attestationsCovid19.data.db.entity.Attestation
import fr.sanchezm.attestationsCovid19.data.db.entity.Profile
import fr.sanchezm.attestationsCovid19.data.repository.AttestationRepository
import fr.sanchezm.attestationsCovid19.data.repository.ConfigRepository
import fr.sanchezm.attestationsCovid19.data.repository.ProfileRepository
import fr.sanchezm.attestationsCovid19.utilities.Event
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*

class AddViewModel(
    private val configRepository: ConfigRepository,
    private val profileRepository: ProfileRepository,
    private val attestationRepository: AttestationRepository,
    private val app: Context
) : ViewModel() {

    private val _errorMessage = MutableLiveData<Event<Int>>()
    private val _dataMessage = MutableLiveData<Event<Int>>()
    private val _startActivity = MutableLiveData<Event<Long>>()

    val errorMessage: LiveData<Event<Int>>
        get() = _errorMessage

    val dataMessage: LiveData<Event<Int>>
        get() = _dataMessage

    val startActivity: LiveData<Event<Long>>
        get() = _startActivity

    // region ViewField
    val firstName = MutableLiveData<String>()
    val lastName = MutableLiveData<String>()
    val birthday = MutableLiveData<String>()
    val birthPlace = MutableLiveData<String>()
    val address = MutableLiveData<String>()
    val city = MutableLiveData<String>()
    val postalCode = MutableLiveData<String>()
    val exitDate = MutableLiveData<String>()
    val exitHour = MutableLiveData<String>()

    val reason1 = MutableLiveData(false)
    val reason2 = MutableLiveData(false)
    val reason3 = MutableLiveData(false)
    val reason4 = MutableLiveData(false)
    val reason5 = MutableLiveData(false)
    val reason6 = MutableLiveData(false)
    val reason7 = MutableLiveData(false)
    val reason8 = MutableLiveData(false)
    val reason9 = MutableLiveData(false)
    // endregion

    private val datePattern = "dd/MM/yyyy"
    private val timePattern = "HH:mm"

    @SuppressLint("LongLogTag")
    fun onGenerateAttestationClick() {
        if (checkAllValue()) {
            val attestation = getAttestation()

            profileRepository.insertProfile(getProfileFromView())
            attestationRepository.addAttestation(attestation)

            _startActivity.value = Event(attestation.createAt)
        } else {
            _errorMessage.value = Event(R.string.error_cannot_create_attestation)
            Log.e("onGenerateAttestationClick", "Cannot generate Attestation")
        }
    }

    fun onBirthdayClick() {
        val c = Calendar.getInstance().also { it.set(1970, 0, 1) }
        if (!birthday.value.isNullOrBlank()) {
            birthday.value?.let { birthday ->
                DateFormat.getDateInstance(DateFormat.SHORT, Locale.FRANCE).parse(birthday)
                    ?.let { c.time = it }
            }
        }
        val year = c.get(Calendar.YEAR)
        val month = c.get(Calendar.MONTH)
        val day = c.get(Calendar.DAY_OF_MONTH)

        val dpd = DatePickerDialog(
            app,
            DatePickerDialog.OnDateSetListener { _, yearPicked, monthOfYear, dayOfMonth ->
                birthday.value =
                    "${getFormattedDayOrMonth(dayOfMonth)}/${getFormattedDayOrMonth(monthOfYear + 1)}/$yearPicked"
            },
            year,
            month,
            day
        )
        dpd.show()
    }

    fun onExitDateClick() {
        val c = Calendar.getInstance()
        val year = c.get(Calendar.YEAR)
        val month = c.get(Calendar.MONTH)
        val day = c.get(Calendar.DAY_OF_MONTH)

        val dpd = DatePickerDialog(
            app,
            DatePickerDialog.OnDateSetListener { _, yearPicked, monthOfYear, dayOfMonth ->
                exitDate.value =
                    "${getFormattedDayOrMonth(dayOfMonth)}/${getFormattedDayOrMonth(monthOfYear + 1)}/$yearPicked"
            },
            year,
            month,
            day
        )
        dpd.show()
    }

    fun onExitHourClick() {
        val c = Calendar.getInstance()
        val hour = c.get(Calendar.HOUR_OF_DAY)
        val minute = c.get(Calendar.MINUTE)

        val tpd = TimePickerDialog(
            app,
            TimePickerDialog.OnTimeSetListener { _, hourPicked, minutePicked ->
                exitHour.value =
                    "${getFormattedDayOrMonth(hourPicked)}:${getFormattedDayOrMonth(minutePicked)}"
            },
            hour,
            minute,
            true
        )
        tpd.show()
    }

    init {
        setProfileValue()
        setDateHourToday()
    }

    private fun setProfileValue() {
        val profile = profileRepository.getProfile()

        firstName.value = profile.value?.firstName
        lastName.value = profile.value?.lastName
        birthday.value = profile.value?.birthday
        birthPlace.value = profile.value?.birthPlace
        address.value = profile.value?.address
        city.value = profile.value?.city
        postalCode.value = profile.value?.postalCode
    }

    private fun setDateHourToday() {
        exitDate.value = getDateOrTimeFromPattern(datePattern)
        exitHour.value = getDateOrTimeFromPattern(timePattern)
    }

    private fun getDateOrTimeFromPattern(pattern: String): String {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            DateTimeFormatter.ofPattern(pattern).format(LocalDateTime.now())
        } else {
            SimpleDateFormat(pattern, Locale.FRANCE).format(Date())
        }
    }

    private fun getProfileFromView(): Profile {
        return Profile(
            firstName.value.toString(),
            lastName.value.toString(),
            birthday.value.toString(),
            birthPlace.value.toString(),
            address.value.toString(),
            city.value.toString(),
            postalCode.value.toString()
        )
    }

    private fun getAttestation(): Attestation {
        return Attestation(
            getProfileFromView(),
            exitDate.value.toString(),
            exitHour.value.toString(),
            Date().time,
            getReasons()
        )
    }

    private fun getReasons(): List<Int> {
        val reasons = ArrayList<Int>()

        if (reason1.value!!) reasons.add(1)
        if (reason2.value!!) reasons.add(2)
        if (reason3.value!!) reasons.add(3)
        if (reason4.value!!) reasons.add(4)
        if (reason5.value!!) reasons.add(5)
        if (reason6.value!!) reasons.add(6)
        if (reason7.value!!) reasons.add(7)
        if (reason8.value!!) reasons.add(8)
        if (reason9.value!!) reasons.add(9)

        return reasons
    }

    private fun checkAllValue(): Boolean {
        return !firstName.value.isNullOrEmpty()
                && !lastName.value.isNullOrEmpty()
                && !birthday.value.isNullOrEmpty()
                && !birthPlace.value.isNullOrEmpty()
                && !address.value.isNullOrEmpty()
                && !city.value.isNullOrEmpty()
                && !postalCode.value.isNullOrEmpty()
                && !exitDate.value.isNullOrEmpty()
                && !exitHour.value.isNullOrEmpty()
                && checkAtLeastOneReasonSelected()
    }

    private fun checkAtLeastOneReasonSelected(): Boolean {
        return reason1.value!!
                || reason2.value!!
                || reason3.value!!
                || reason4.value!!
                || reason5.value!!
                || reason6.value!!
                || reason7.value!!
                || reason8.value!!
                || reason9.value!!
    }

    private fun getFormattedDayOrMonth(date: Int): String {
        var formattedDate: String = date.toString()

        if (formattedDate.length <= 1) {
            formattedDate = "0$formattedDate"
        }
        return formattedDate
    }

}